#!/bin/sh
##
##  gencert.sh -- Create self-signed test certificate
##  Christian Zoffoli <czoffoli@linux-mandrake.com> 
##  Version 0.2 - 20010501
##  Alexander Belov <asnel@mail.ru>
##  Version 0.3 - 20040122


### external tools
openssl="/usr/bin/openssl"
expect="/usr/bin/expect"

### Certificate file name
CERT=cert

### Owner of certifacates
USER=root
GROUP=root

[ "$1" ] && CERT=$1

CERTFILE="`basename $CERT`"

#   find some random files
#   (do not use /dev/random here, because this device 
#   doesn't work as expected on all platforms)
randfiles=''
for file in /var/log/messages /bin/sh /etc/localtime /var/log/wtmp /etc/hosts /etc/resolv.conf; do
    if [ -f $file ]; then
        if [ ".$randfiles" = . ]; then
            randfiles="$file"
        else
            randfiles="${randfiles}:$file"
        fi
    fi
done


[ -e $CERT.cert ] || [ -e $CERT.key ] && \
    echo "$CERT.cert or $CERT.key already exist, dying" && exit 1
[ ! -x $openssl ] || [ ! -x $expect ] &&
    echo "$openssl or $expect don't exist, dying" && exit 1

mkdir -p /tmp/tmpssl-$$ || exit 1
pushd /tmp/tmpssl-$$ > /dev/null

COMMONNAME="`hostname --long`"
[ -n "$COMMONNAME" ] || COMMONNAME="www.openssl.org"

EMAIL="`id -un`@$COMMONNAME"

LANG='US'
[ -f /etc/sysconfig/i18n ] && source /etc/sysconfig/i18n
[ -n "$LANG" ] && LANG=`echo $LANG | sed -e 's/.._//' -e 's/\..*//' | tr a-z A-Z`

[ -f /etc/sysconfig/clock ] && source /etc/sysconfig/clock
[ -n $ZONE ] && CITY="`echo $ZONE | sed -e 's/[^\/]*\///'`"

cat >.cfg <<EOF
[ req ]
default_bits                    = 1024
distinguished_name              = req_DN
RANDFILE                        = ca.rnd
[ req_DN ]
countryName                     = "1. Country Name             (2 letter code)"
countryName_default             = "$LANG"
countryName_min			= 2
countryName_max			= 2
stateOrProvinceName		= "2. State or Province Name   (full name)    "
stateOrProvinceName_default	= ""
localityName                    = "3. Locality Name            (eg, city)     "
localityName_default		= "$CITY"
0.organizationName		= "4. Organization Name        (eg, company)  "
0.organizationName_default	= ""
organizationalUnitName		= "5. Organizational Unit Name (eg, section)  "
organizationalUnitName_default	= ""
commonName			= "6. Common Name              (eg, CA name)  "
commonName_max			= 64
commonName_default		= "$COMMONNAME"
emailAddress			= "7. Email Address            (eg, name@FQDN)"
emailAddress_max		= 40
emailAddress_default		= "$EMAIL"
EOF

cat >script.exp <<EOF
spawn $openssl req -config .cfg -new  -rand $randfiles -x509 -nodes -out $CERTFILE.cert -keyout $CERTFILE.key -days 999999

expect "Country Name"
send -- "\r"
expect "State or Province Name"
send -- "\r"
expect "Locality Name"
send -- "\r"
expect "Organization Name"
send -- "\r"
expect "Organizational Unit Name"
send -- "\r"
expect "Common Name"
send -- "\r"
expect "Email Address"
send -- "\r"
expect eof
EOF

$expect -f ./script.exp > /dev/null 2>&1
if [ $? -ne 0 ]; then
       	echo "cca:Error: Failed to generate certificate " 1>&2
	exit 1
fi


popd >/dev/null


rm -f /tmp/tmpssl-$$/*.csr
rm -f /tmp/tmpssl-$$/ca.*
chmod 400 /tmp/tmpssl-$$/*

pushd "`dirname $CERT`" > /dev/null
cp /tmp/tmpssl-$$/$CERTFILE.* .
chown $USER:$GROUP $CERTFILE.cert
chown $USER:$GROUP $CERTFILE.key

rm -rf /tmp/tmpssl-$$
popd > /dev/null
